package com.brainysoftware.batik;

import java.io.File;
import java.io.IOException;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.batik.dom.svg.SVGDOMImplementation;
import org.apache.batik.transcoder.TranscoderException;
import org.apache.batik.transcoder.TranscoderInput;
import org.apache.batik.transcoder.TranscoderOutput;
import org.apache.batik.transcoder.image.PNGTranscoder;
import org.w3c.dom.CDATASection;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

public class DisplayCodeServlet extends HttpServlet {
	private static final long serialVersionUID = 1L;
	
	private final String svgNS = SVGDOMImplementation.SVG_NAMESPACE_URI;
	
	public void doGet(HttpServletRequest req, HttpServletResponse resp) throws IOException {
		
		// Get the check number from the session
		HttpSession session = req.getSession();
		String code = (String) session.getAttribute("number");
		
		// Create an SVG document and set its size
		DOMImplementation dom = SVGDOMImplementation.getDOMImplementation();
		Document doc = dom.createDocument(svgNS, "svg", null);
		
		Element root = doc.getDocumentElement();
		root.setAttributeNS(null, "width", "300");
		root.setAttributeNS(null, "height", "100");
		
		// Add an SVG font reference
		ServletContext ctx  = getServletContext();
		File fontFile = new File(ctx.getRealPath("/ForgottenFuturist.svg"));
		Element defs = doc.createElementNS(svgNS, "defs");
		Element style = doc.createElementNS(svgNS, "style");
		style.setAttributeNS(null, "type", "text/css");
		CDATASection fontRef = doc.createCDATASection(
				"@font-face { font-family: 'ForgottenFuturist'; src: url(\"" + 
				fontFile.toURL().toString() + 
				"#ForgottenFuturist\") format(svg) }");
		style.appendChild(fontRef);
		defs.appendChild(style);
		
		// Create a filter for the background
		Element bckgFilter = doc.createElementNS(svgNS, "filter");
		bckgFilter.setAttributeNS(null, "id", "BackgroundFilter");
		
		Element turbulence = doc.createElementNS(svgNS, "feTurbulence");
		turbulence.setAttributeNS(null, "type", "turbulence");
		turbulence.setAttributeNS(null, "baseFrequency", "0.05");
		turbulence.setAttributeNS(null, "numOctaves", "3");
		
		// Create a filter for the code
		Element codeFilter = doc.createElementNS(svgNS, "filter");
		codeFilter.setAttributeNS(null, "id", "CodeFilter");
		
		Element blur = doc.createElementNS(svgNS, "feGaussianBlur");
		blur.setAttributeNS(null, "in", "SourceGraphic");
		blur.setAttributeNS(null, "stdDeviation", "2");
		blur.setAttributeNS(null, "result", "blur");
		
		codeFilter.appendChild(blur);
		defs.appendChild(codeFilter);
		
		bckgFilter.appendChild(turbulence);
		defs.appendChild(bckgFilter);
		root.appendChild(defs);
		
		// This rectangle will serve as a background
		Element rect = doc.createElementNS(svgNS, "rect");
		rect.setAttributeNS(null, "x", "0");
		rect.setAttributeNS(null, "y", "0");
		rect.setAttributeNS(null, "width", "300");
		rect.setAttributeNS(null, "height", "100");
		rect.setAttributeNS(null, "filter", "url(#BackgroundFilter)");
		
		// Define random shifts in vertical direction
		StringBuffer yValues = new StringBuffer();
		
		for (int i = 0; i < 8; i ++) {
			int delta = (int) Math.round(50 * Math.random()) - 25;
			if (i != 0) yValues.append(" ");
			yValues.append(65 + delta);
		}
		
		// Define random rotations
		StringBuffer rotations = new StringBuffer();
		
		for (int i = 0; i < 8; i ++) {
			int delta = (int) Math.round(60 * Math.random()) - 30;
			if (i != 0) rotations.append(" ");
			rotations.append(delta);
		}
		
		// Create the text element
		Element text = doc.createElementNS(svgNS, "text");
		Text greetingText = doc.createTextNode(code);
		text.appendChild(greetingText);
		text.setAttributeNS(null, "x", "15");
		text.setAttributeNS(null, "y", yValues.toString());
		text.setAttributeNS(null, "rotate", rotations.toString());
		text.setAttributeNS(null, "font-size", "45");
		text.setAttributeNS(null, "fill", "indigo");
		text.setAttributeNS(null, "font-family", "ForgottenFuturist");
		text.setAttributeNS(null, "letter-spacing", "7%");
		text.setAttributeNS(null, "filter", "url(#CodeFilter)");
		
		root.appendChild(rect);
		root.appendChild(text);
		
		// Transcode SVG to PNG and send it as a response
		resp.setContentType("image/png");
		
		PNGTranscoder t = new PNGTranscoder();
		TranscoderInput input =	new TranscoderInput(doc);		
		TranscoderOutput output = new TranscoderOutput(resp.getOutputStream());
		try {
			t.transcode(input, output);
		}
		catch (TranscoderException te) {
			te.printStackTrace();
		}
    		resp.getOutputStream().close();
	}
	
	public void doPost(HttpServletRequest req, HttpServletResponse resp) throws IOException {
		doGet(req, resp);
	}
}
